<?php 

if(!defined('ABSPATH')) exit;

class Firstdata_Api {
	public static $errors;
	static $binKey;
	const firstdata_form = 'https://vpos.polcard.com.pl/vpos/ecom/cardTokenization.htm';
	const firstdata_endpoint1 = 'https://wtxn.polcard.com.pl:8443/card/tokenizer/v1/TokenService?wsdl';
	const firstdata_endpoint2 = 'https://wtxn.polcard.com.pl:8443/pos/virtual/transaction/v1/TransactionService?wsdl';
	const firstdata_repeat = array(-3, -1, 24);
	const prefix = 'WC_JPW_';
	
	static function registerPretoken($order_id) {
		$params = array(
		    'tokenRequestor' => get_option('firstdata_token_requestor'),
		    'returnOtc' => true,
		    'externOrderIdent' => self::prefix . $order_id
		);

		$response = self::call1("registerPretoken", $params);
		if(!$response) return null;
		error_log('registerPretoken resp ' . print_r($response, true));
		return $response->correlationIdent;
	}
	
	static function getTokenData($order_id) {
	    $params = array(
	        'tokenRequestor' => get_option('firstdata_token_requestor'),
	        'externOrderIdent' => self::prefix . $order_id
	    );
	    
	    $response = self::call1("getTokenizationData", $params);
	    if(!$response) return null;
	    error_log('getTokenizationData resp ' . print_r($response, true));
	    if(($response->tokenizationStatus != 4) || ($response->tokenStatus != 1)) return null;
	    return array($response->tokenNumber, $response->tokenExpiryDate);
	}
	
	static function doTokenPay($order, $token, $renew = false) {
	    $order_data = $order->get_data();
	    $date = '20' . substr($token[1], 0, 2) . '-' . substr($token[1], 2, 2);
	    $params = array(
	        'merchantCode' => get_option('firstdata_mid'),
	        'transaction' => array(
	            'recurringPayment' => array(
	                'paymentInstrument' => array(
	                    'number' => $token[0],
    	                'expirationDate' => $date
                    ),
	                'payment' => array(
	                    'externPaymentCode' => self::prefix . $order->get_id() . ($renew ? '_R' . $order->get_meta('firstdata_recurring_renew_count') : ''),
	                    'txnAmount' => array(
	                        'amount' => round(($renew ? $order->get_meta('firstdata_recurring_renew_amount') : $order->get_meta('firstdata_recurring_amount')) * 100),
	                        'currencyCode' => get_woocommerce_currency()
	                    ),
	                    'merchantLabel' => substr(get_bloginfo('name'), 0, 30),
	                    'test' => get_option('firstdata_test') == 'yes' ? 1 : 0,
	                    'emailAddress' => $order_data['billing']['email']
	                )
	            )
	        ),
	    );
	    error_log('authorizePayment req ' . print_r($params, true));
	    $response = self::call2("authorizePayment", $params);
	    if(!$response) return null;
	    error_log('authorizePayment resp ' . print_r($response, true));
	    return isset($response->return) ? $response->return : $response->WebServiceException;
	}
	
	static function call1($meth, $params) {
	    $url = self::firstdata_endpoint1;
		$user = get_option('firstdata_mid') . '.' . get_option('firstdata_vpos_user');
		$pass = get_option('firstdata_vpos_pass');
		error_log("URL=$url user=$user pass=$pass");
		try {
			error_log($meth . ' req ' . print_r($params, true));
			$option = array(
			    'trace' => 1,
			    'login' => $user,
			    'password' => $pass
			);
			$client = new SoapClient($url, $option);
				
			$response = $client->__soapCall($meth, array($params));
		} catch (Exception $e) {
			self::$errors = $e->getMessage();
			error_log('ERR[1]=' . $e->getMessage());
			error_log('resp ' . print_r($response, true));
			return false;
		}
		return $response;
	}
	
	static function call2($meth, $params) {
	    $url = self::firstdata_endpoint2;
	    $user = get_option('firstdata_mid') . '.' . get_option('firstdata_vpos_user');
	    $pass = get_option('firstdata_vpos_pass');
	    error_log("URL=$url user=$user pass=$pass");
	    try {
	        error_log($meth . ' req ' . print_r($params, true));
	        $option = array(
	            'trace' => 1,
	            'login' => $user,
	            'password' => $pass
	        );
	        $client = new SoapClient($url, $option);
	        
	        $response = $client->__soapCall($meth, array($params));
	    } catch (SoapFault $e) {
	        self::$errors = $e->getMessage();
	        error_log('ERR[2.1]=' . $e->getMessage());
	        error_log('resp ' . print_r($e->detail->WebServiceException, true));
	        return $e->detail;
	    } catch (Exception $e) {
	        self::$errors = $e->getMessage();
	        error_log('ERR[2.2]=' . $e->getMessage());
	        return false;
	    }
	    return $response;
	}
	
	static function doRedir($corid, $order) {
	    $order_data = $order->get_data();
	    $data = array(
	        'merchant_code' => get_option('firstdata_mid'),
	        'order_id' => self::prefix . $order->get_id(),
	        'correlation_id' => $corid,
	        'language' => 'pl',
	        'email' => $order_data['billing']['email'],
	        'cardholder_firstname' => $order_data['billing']['first_name'],
	        'cardholder_lastname' => $order_data['billing']['last_name'],
	        'merchant_label' => substr(get_bloginfo('name'), 0, 30),
	        'with_txn' => 'false',
	        'return_url' => add_query_arg('a', 'd', WC()->api_request_url('Firstdata_Recurring_Gateway')),
	        'notification_url' => add_query_arg('a', 'i', WC()->api_request_url('Firstdata_Recurring_Gateway')),
	        'error_return_url' => add_query_arg('a', 'e', WC()->api_request_url('Firstdata_Recurring_Gateway')),
	        'test' => get_option('firstdata_test') == 'yes' ? 'true' : 'false',
	        'payment_context' => 'REC'
	    );
	    error_log('doRedir ' . print_r($data, true));
	    ?><!DOCTYPE HTML><html <?php language_attributes() ?>><head><meta charset="<?php bloginfo('charset') ?>" /><?php wp_head() ?></head><body><br/><br/><br/><h3 style="text-align: center;"><?= __('Redirecting to FirstData...', 'firstdata_recurring') ?></h3><?php
	    ?><form action="<?= self::firstdata_form ?>" method="post" id="firstdata_payment_form"><?php
	    foreach($data as $k => $v) self::genHidden($k, $v);
	    self::genHidden('controlData', self::genControlData($data));
        ?><script type="text/javascript">document.getElementById('firstdata_payment_form').submit();</script></form><?php wp_footer() ?></body></html><?php
	}

	static function validIpn() {
	    if(empty($_POST)) return 'E-40';
	    $data = array(
	        'merchant_code' => isset($_POST['merchant_code']) ? $_POST['merchant_code'] : '',
	        'order_id' => isset($_POST['order_id']) ? $_POST['order_id'] : '',
	        'correlation_id' => isset($_POST['correlation_id']) ? $_POST['correlation_id'] : '',
	        'payment_code' => isset($_POST['payment_code']) ? $_POST['payment_code'] : '',
	        'response_code' => isset($_POST['response_code']) ? $_POST['response_code'] : ''
	    );
	    if(!isset($_POST['controlData']) || $_POST['controlData'] != self::genControlData($data)) return 'E-41';
	    if($data['merchant_code'] != get_option('firstdata_mid')) return 'E-42';
	    $tmp = explode('_', $data['order_id']);
	    if((count($tmp) != 3) || ($tmp[0] != 'WC')) return 'E-43';
	    $order = new WC_Order($tmp[2]);
	    if(!$order) return 'E-44';
	    if(($data['correlation_id'] != '') && ($order->get_meta('firstdata_recurring_corrid') != $data['correlation_id'])) return 'E-45';
	    return $data;
	}
	
	static function validError() {
	    if(empty($_POST)) return 'E-60';
	    $data = array(
	        'merchant_code' => isset($_POST['merchant_code']) ? $_POST['merchant_code'] : '',
	        'order_id' => isset($_POST['order_id']) ? $_POST['order_id'] : '',
	        'correlation_id' => isset($_POST['correlation_id']) ? $_POST['correlation_id'] : '',
	        'message' => isset($_POST['message']) ? $_POST['message'] : '',
	        'err_code' => isset($_POST['err_code']) ? $_POST['err_code'] : ''
	    );
	    if(!isset($_POST['controlData']) || $_POST['controlData'] != self::genControlData($data)) return 'E-61';
	    if($data['merchant_code'] != get_option('firstdata_mid')) return 'E-62';
	    $tmp = explode('_', $data['order_id']);
	    if((count($tmp) != 3) || ($tmp[0] != 'WC')) return 'E-63';
	    $order = new WC_Order($tmp[2]);
	    if(!$order) return 'E-64';
	    if(($data['correlation_id'] != '') && ($order->get_meta('firstdata_recurring_corrid') != $data['correlation_id'])) return 'E-65';
	    return $data;
	}
	
	static function genHidden($k, $v) {
	    ?><input type="hidden" name="<?= $k ?>" value="<?= $v ?>"/><?php
	}
	
	static function getBinKey() {
	    if(self::$binKey == null) self::$binKey = hex2bin(get_option('firstdata_key'));
	    return self::$binKey;
	}

	static function genControlData($data) {
	    $tmp = array();
	    foreach ($data as $k => $v)
	        if(strlen($v) > 0)
    	        $tmp[] = $k . '=' . $v;
        return hash('sha256', join('&' , $tmp) . self::getBinKey());
	}

}

?>