<?php 

if(!defined('ABSPATH')) exit;

class Firstdata_Recurring_Gateway extends WC_Payment_Gateway {
	
	public function __construct() {
		$this->id = 'firstdata_recurring';
		$this->has_fields = false;
		$this->method_title = 'Firstdata Recurring Payments';
		$this->method_description = 'Firstdata gateway for recurring payments';
		$this->init_form_fields();
		$this->init_settings();
		$this->init();
		$this->supports = array('subscriptions');
	}
	
	public function init() {
	    $this->title = $this->get_option('title');
	    $this->description = $this->get_option('description');
	    if(!$this->title) $this->title = $this->method_title;
	    if(!$this->description) $this->description = $this->method_description;
	    add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ));
	    add_action( 'woocommerce_api_firstdata_recurring_gateway', array($this, 'api_cb'));
	}

	public function init_form_fields() {
	    $this->form_fields = array(
	        'enabled' => array(
	            'title' => __( 'Enable/Disable', 'woocommerce' ),
	            'type' => 'checkbox',
	            'label' => __( 'Enable Firstdata Recurring Payments', 'firstdata_recurring' ),
	            'default' => 'yes'
	        ),
	        'title' => array(
	            'title' => __( 'Title', 'woocommerce' ),
	            'type' => 'text',
	            'description' => __( 'This controls the title which the user sees during checkout.', 'woocommerce' ),
	            'default' => __( 'Firstdata Recurring Payments', 'firstdata_recurring' ),
	            'desc_tip'      => true,
	        ),
	        'description' => array(
	            'title' => __( 'Customer Message', 'woocommerce' ),
	            'type' => 'textarea',
	            'default' => ''
	        )
	    );
	}

	function process_payment($order_id) {
	    global $woocommerce;
	    
	    if(!self::have_yws() && !self::have_jpw()) {
	        wc_add_notice( __('Order must have a subscription product for this payment method', 'firstdata_recurring'), 'error' );
	        return;
	    }
	    
	    $type = 0;
	    $order = new WC_Order($order_id);
	    if(self::have_yws()) {
	        $yws_type = ywsbs_is_an_order_with_subscription($order_id);
	        if(!$yws_type) {
    	        wc_add_notice( __('Order must have a subscription product for this payment method', 'firstdata_recurring'), 'error' );
	            return;
	        }
	        if($yws_type == 'parent') $type = 1;
	        elseif($yws_type == 'renew') $type = 2;
	    }
	    
	    if(self::have_jpw()) {
	        if(!is_jpw_subscribe()) {
	            wc_add_notice( __('Order must have a subscription product for this payment method', 'firstdata_recurring'), 'error' );
	            return;
	        }
	        $type = 1;
	    }
	    
	    if(!$type) {
	        error_log('Nierozpoznane żądanie płatności');
	        wc_add_notice( __('Payment request not recognized', 'firstdata_recurring'), 'error' );
	        return;
	    }
	    
	    $currency = $order->get_currency();
	    if($currency != 'PLN') {
	        error_log('Waluta niedozwolona: ' . $currency);
	        wc_add_notice( __('Currency not allowed for this payment method', 'firstdata_recurring'), 'error' );
	        return;
	    }
	    
//	    error_log('get='.print_r($_GET, true));
//	    error_log('post='.print_r($_GET, true));
	    $token = $this->getToken($order);
	    if(!$token) {
	        $corid = Firstdata_Api::registerPretoken($order_id);
    	    if(!$corid) {
	            wc_add_notice( __('Payment error: ', 'firstdata_recurring') . Firstdata_Api::$errors, 'error' );
	            return;
	        }
	    
    	    $order->update_meta_data('firstdata_recurring_corrid', $corid);
    	    $order->save();
    	    $args = array(
	            'a' => 'r',
	            'id' => $order_id,
    	        'c' => $corid,
	            't' => hash('sha256', $order_id . '&' . $corid . Firstdata_Api::getBinKey())
	        );
	        return array(
    	        'result' => 'success',
    	        'redirect' => add_query_arg($args , WC()->api_request_url('Firstdata_Recurring_Gateway'))
	        );
	    }
	    else {
	        $txn_info = Firstdata_Api::doTokenPay($order, $token);
	        if(($txn_info != null) && isset($txn_info->responseCode) && $txn_info->responseCode == '00') {
	            $this->doComplete($order, $txn_info->paymentCode);
	        }
	        else {
	            $this->doFail($order, 'P' . (isset($txn_info->responseCode) ? $txn_info->responseCode : isset($txn_info->detailedCode) ? $txn_info->detailedCode : ''));
	        }
	    }
	}
	
	private function getToken($order) {
	    if(self::have_yws()) {
	        $yws_type = ywsbs_is_an_order_with_subscription($order->get_id());
    	    if($type == 'renew') {
    	        $parent_order = new WC_Order($order->get_parent_id());
    	        if($parent_order->get_meta('firstdata_recurring_token_number'))
    	            return Firstdata_Api::getTokenData($parent_order->get_id());
//        	        return array($parent_order->get_meta('firstdata_recurring_token_number'), $parent_order->get_meta('firstdata_recurring_token_exp'));
    	        else
    	            return null;
    	    }
	    }
	    if($order->get_meta('firstdata_recurring_token_number'))
	        return Firstdata_Api::getTokenData($order->get_id());
    	    //return array($order->get_meta('firstdata_recurring_token_number'), $order->get_meta('firstdata_recurring_token_exp'));
	    else
	        return null;
	}

	public function api_cb() {
	    $action = '';
	    error_log('api cb get=' . print_r($_GET, true) . ' post=' . print_r($_POST, true));
	    if(isset($_GET['a'])) $action = $_GET['a'];
	    switch ($action) {
	        case 'r':
	            if(isset($_GET['id']) && isset($_GET['c']) && isset($_GET['t']) && ($_GET['t'] == hash('sha256', $_GET['id'] . '&' . $_GET['c'] . Firstdata_Api::getBinKey()))) {
	                $this->createRedir($_GET['id'], $_GET['c']);
    	            exit;
	            }
	            else {
	                error_log('cb err E-20');
    	            wp_die(__('Invalid request','firstdata_recurring') . ' [E-20]');
	            }
	        case 'i':
	            $data1 = Firstdata_Api::validIpn();
	            if(gettype($data1) == 'string') $data2 = Firstdata_Api::validError();
	            else {
	                $this->doIpn($data1);
	                echo "OK";
	                exit;
	            }
	            if(gettype($data2) == 'string') {
	                error_log('cb err ' . $data1 . '/' . $data2);
	                wp_die(__('Invalid request','firstdata_recurring') . ' [' . $data1 . '/' . $data2 . ']');
	            }
	            else {
	                $this->doError($data2);
	                echo "OK";
	                exit;
	            }
	        case 'd':
	            $data = Firstdata_Api::validIpn();
	            if(gettype($data) == 'string') {
	                error_log('cb err ' . $data);
	                wp_die(__('Invalid request','firstdata_recurring') . ' [' . $data . ']');
	            }
	            else {
	                $this->doIpn($data, true);
	                exit;
	            }
	        case 'e':
	            $data = Firstdata_Api::validError();
	            if(gettype($data) == 'string') {
	                error_log('cb err ' . $data);
	                wp_die(__('Invalid request','firstdata_recurring') . ' [' . $data . ']');
	            }
	            else {
	               $this->doError($data, true);
                   exit;
                }
	    }
	    wp_die(__('Invalid request','firstdata_recurring') . ' [E-01]');
	}

	function createRedir($order_id, $corid) {
	    $order = new WC_Order($order_id);
	    $amount = 0;
	    $renew_amount = 0;
	    foreach ($order->get_items() as $item) {
	        $product = wc_get_product($item['product_id']);
	        if(self::have_jpw() && $product->is_type('paywall_subscribe')) {
	            $amount += $product->get_price_including_tax();
	            $renew_amount = $product->get_price_including_tax();
	        }
	        elseif(self::have_yws() && YITH_WC_Subscription()->is_subscription($item['product_id'])) {
	            $amount += $product->get_price_including_tax() * $item->get_quantity();
	        }
	        else {
	            $amount += $product->get_price_including_tax() * $item->get_quantity();
	        }
	    }
	    $order->update_meta_data('firstdata_recurring_amount', $amount);
	    $order->update_meta_data('firstdata_recurring_renew_amount', $renew_amount);
	    $order->save();
	    Firstdata_Api::doRedir($corid, $order);
	}

	private function doRenew($order_data) {
	    $order = new WC_Order($order_data->ID);
	    $renew_next = $order->get_meta('firstdata_recurring_next_try');
	    $renew_amount = $order->get_meta('firstdata_recurring_renew_amount');
	    $renew_count = $order->get_meta('firstdata_recurring_renew_count');
	    $renew_count++;
	    $order->update_meta_data('firstdata_recurring_renew_count', $renew_count);
	    $order->save();
	    error_log('doRenew o=' . $order_data->ID . ' k=' . $renew_amount . ' d=' . date("Y-m-d H:i:s", $renew_next));
	    $token = $this->getToken($order);
	    if(!$token) {
	        error_log('doRenew bez tokenu');
	    }
	    else {
	        $txn_info = Firstdata_Api::doTokenPay($order, $token, true);
	        if(($txn_info != null) && isset($txn_info->responseCode) && $txn_info->responseCode == '00') {
	            $this->doComplete($order, $txn_info->paymentCode);
	        }
	        else {
	            $this->doFail($order, 'P' . (isset($txn_info->responseCode) ? $txn_info->responseCode : isset($txn_info->detailedCode) ? $txn_info->detailedCode : ''));
	        }
	    }
	    
	}

	function doIpn($data, $redir = false) {
	    $code = $data['response_code'];
	    $tmp = explode('_', $data['order_id']);
	    $order = new WC_Order($tmp[2]);
	    
	    if(!$order->has_status(array('processing', 'completed'))) {
	        if($code == '00') {
	            $order->update_meta_data('firstdata_recurring_tokeninzation_txn_id', $txn_id);
	            $token = Firstdata_Api::getTokenData($order->get_id());
	            if($token) {
	                $order->update_meta_data('firstdata_recurring_token_number', $token[0]);
	                $order->update_meta_data('firstdata_recurring_token_exp', $token[1]);
	                $txn_info = Firstdata_Api::doTokenPay($order, $token);
	                if(($txn_info != null) && isset($txn_info->responseCode) && $txn_info->responseCode == '00') {
	                    $this->doComplete($order, $txn_info->paymentCode);
	                }
	                else {
	                    $this->doFail($order, 'P' . (isset($txn_info->responseCode) ? $txn_info->responseCode : isset($txn_info->detailedCode) ? $txn_info->detailedCode : ''));
	                }
	            }
	            else {
	                $this->doFail($order, 'T');
	            }
	            //    	    if($redir) wc_add_notice(__( 'Payment registered.', 'firstdata_recurring'));
	        }
	        else {
	            $this->doFail($order, $code);
	            //	        if($redir) wc_add_notice(__( 'Card registration failed.', 'firstdata_recurring'), 'error');
	        }
	    }
	    if($redir) wp_safe_redirect($this->get_return_url($order));
	}

	function doError($data, $redir = false) {
	    $code = $data['err_code'];
	    $tmp = explode('_', $data['order_id']);
	    $order = new WC_Order($tmp[2]);
        $this->doFail($order, 'E' . $code, $data['message']);
        if($redir) {
//            if($redir) wc_add_notice(__( 'Payment failed.', 'firstdata_recurring'), 'error');
            wp_safe_redirect($this->get_return_url($order));
        }
	}
	
	private function doComplete($order, $txn_id) {
	    global $woocommerce;

	    error_log('doComplete id=' . $order->get_id() . ' s=' . $order->get_status() . ' t=' . $txn_id);
	    if(self::have_jpw()) {
	        $prod = $this->get_jpw_prod($order);
	        if($prod == null) {
	            error_log('Nie znaleziono produktu paywall_subscribe!');
	            return;
	        }
	        $days = $this->get_jpw_days($prod);
	        $user_id = $order->get_user_id();
	        $next = $order->get_meta('firstdata_recurring_next_payment');
	        if(time() > $next) $base = time();
	        else $base = $next;
	        $time = $base + $days * 86400;
	        $ftime = date('Y-m-d H:i:s', $time);
	        update_user_option($user_id, 'jpw_subscribe_status', 'ACTIVE');
	        update_user_option($user_id, 'jpw_expired_date', $ftime);
	        if($order->has_status(array('processing', 'pending', 'on-hold', 'failed'))) {
	            $order->add_order_note('Registered payment ' . $txn_id);
	            $order->payment_complete($txn_id);
	            $order->update_meta_data('firstdata_recurring_txn_id', $txn_id);
	            $order->update_meta_data('firstdata_recurring_next_payment', $time);
	            $order->update_meta_data('firstdata_recurring_next_try', $time + Firstdata_Api::firstdata_repeat[0] * 3600);
	            $order->update_meta_data('firstdata_recurring_tfail_count', 0);
	            $order->save();

	            if($woocommerce->cart != null) $woocommerce->cart->empty_cart();
	        }
	    }
	    if(self::have_yws()) {
	        if($order->has_status(array('pending', 'on-hold', 'failed'))) {
    	        $order->add_order_note('Registered payment ' . $txn_id);
	            $order->payment_complete($txn_id);
	            $order->update_meta_data('firstdata_recurring_txn_id', $txn_id);
	            $order->save();

    	        $woocommerce->cart->empty_cart();
    	    }
	    }
	}

	private function doFail($order, $code, $message = null) {
	    error_log('doFail id=' . $order->get_id() . ' s=' . $order->get_status() . ' c=' . $code . ($message ? ' m=' . $message : ''));
	    if($code == 'P70') return;
	    if(self::have_jpw()) {
	        $user_id = $order->get_user_id();
//	        update_user_option($user_id, 'jpw_subscribe_status', false);
//	        update_user_option($user_id, 'jpw_expired_date', false);	        
	        if($order->has_status(array('pending', 'on-hold'))) {
	            $order->update_status('failed');
	            $order->add_order_note('Payment fail: code ' . $code . ($message ? ' ' . $message : ''));
	            $order->update_meta_data('firstdata_recurring_tfail_code', $code);
	            $count = $order->get_meta('firstdata_recurring_tfail_count');
	            $count++;
	            $order->update_meta_data('firstdata_recurring_tfail_count', $count);
	            $order->save();
	        }
	    }
	    if(self::have_yws()) {
	        if($order->has_status(array('pending', 'on-hold'))) {
        	    $order->update_status('failed');
    	        $order->add_order_note('Payment fail: code ' . $code . ($message ? ' ' . $message : ''));
    	        $order->update_meta_data('firstdata_recurring_tfail_code', $code);
        	    $count = $order->get_meta('firstdata_recurring_tfail_count');
                $count++;
        	    $order->update_meta_data('firstdata_recurring_tfail_count', $count);
        	    $order->save();
    	    }
	    }
	}
		
	private function get_jpw_prod($order) {
	    $prod = null;
	    foreach($order->get_items() as $item) {
	        $product = wc_get_product($item['product_id']);
	        if($product->is_type('paywall_subscribe')) $prod = $product;
	    }
	    return $prod;
	}

	private function get_jpw_days($prod) {
	    $interval = strtolower($prod->get_interval());
	    $duration = $prod->get_duration();
	    switch($interval) {
	        case 'day':
	            $days = $duration;
	            break;
	        case 'week':
	            $days = $duration * 7;
	            break;
	        case 'month':
	            $days = $duration * 30;
	            break;
	        case 'year':
	            $days = $duration * 365;
	            break;
	        default:
	            $days = 0;
	            break;
	    }
	    return $days;
	}

	public function doCron() {
	    $args = array(
	        'post_type' => 'shop_order',
	        'post_status' => 'wc-processing',
	        'numberposts' => 20,
	        'orderby' => 'rand',
	        'meta_query' => array(
	            array(
	                'key' => 'firstdata_recurring_next_try',
	                'value' => time(),
	                'compare' => '<',
	            )
	        )
	    );
	    $orders = get_posts($args);
	    foreach($orders as $order)
    	    $this->doRenew($order);
	}

	static function have_yws() {
	    return function_exists('ywsbs_is_an_order_with_subscription');
	}

	static function have_jpw() {
	    return function_exists('is_jpw_subscribe');
	}
	
}

?>