<?php

/**
* Plugin Name: Firstdata Recurring
* Developer: Marcin Bogdański
* Author: Marcin Bogdański (Opennet)
* Author URI: http://opennet.pl/
* Version: 1.0.1
* Requires at least: 4.7
* Tested up to: 5.5
* Requires PHP: 5.3
* Description: Firstdata recurring payments gateway
*/

if(!defined('ABSPATH')) exit;
//include 'conf.php';

class FirstdataRecurringPlugin {

    const CRON_HOOK = 'do_cron';
    public static $instance;
    private $methods = array();
	
	public function __construct() {
		add_action('plugins_loaded', array($this, 'init'), 100);
		add_action(self::CRON_HOOK, array($this, 'doCron'));
	}
	
	public static function FirstdataRecurringPlugin() {
		if (self::$instance === null) {
			self::$instance = new self();
		}
		return self::$instance;
	}
	
	public function init() {
	    if( class_exists( 'WC_Payment_Gateway' ) ) {
    		include 'classes/method.php';
	   	    include 'classes/api.php';
		    $this->init_methods();
		    add_filter('woocommerce_get_settings_pages', array($this, 'woocommerce_get_settings_pages'));	
            add_filter('woocommerce_payment_gateways', array( $this, 'woocommerce_payment_gateways'));
            add_filter('woocommerce_available_payment_gateways', array( $this, 'woocommerce_available_payment_gateways'), 100);
            add_filter('plugin_action_links_' . plugin_basename(__FILE__), array($this, 'linksFilter'));
            add_filter('woocommerce_account_menu_items', array($this, 'fdp_add_user_menu'));
            add_action('init', array($this, 'plugin_endpoint'));
            add_action('woocommerce_account_jnews-fdp_endpoint', array($this, 'list_my_subs'));
            add_action('woocommerce_account_action-fdp_endpoint', array($this, 'sub_cancel'));
            add_filter('wc_get_template', array($this, 'fdp_get_template'), 10, 2);
	    }
	}

	function fdp_add_user_menu($items) {
	    $items = array_slice($items, 0, 5, true) + array('jnews-fdp' => 'Moje subskrypcje JNews') + array_slice($items, 5, NULL, true);
	    return $items;
	}

	static function s_plugin_endpoint() {
	    add_rewrite_endpoint( 'jnews-fdp', EP_PAGES );
	    add_rewrite_endpoint( 'action-fdp', EP_PAGES );
	}

	function plugin_endpoint() {
	    self::s_plugin_endpoint();
	}

	function list_my_subs() {
	    $args = array(
	        'post_type' => 'shop_order',
	        'orderby' => 'date',
	        'order' => 'DESC',
	        'post_status' => array('wc-processing', 'wc-completed'),
	        'meta_query' => array(
	            array(
	                'key' => '_customer_user',
	                'value' => '' . get_current_user_id(),
	                'compare' => '=',
	            )
	        )
	    );
//	    error_log(print_r($args,true));
	    $customer_orders = get_posts($args);
//	    error_log(print_r($customer_orders,true));
	    
	    wc_print_notices();
	    wc_get_template(
	        'subs.php',
	        array(
	            'customer_orders' => $customer_orders,
	            'has_items'       => 0 < count($customer_orders),
	            'list_cols'       => array(
	                'sub-start'   => 'Start',
	                'sub-count'   => 'Liczba odnowień',
	                'sub-next'    => 'Kolejne odnowienie',
                    'sub-amount'  => 'Kwota',
	                'sub-status'  => 'Status',
	                'sub-actions' => 'Akcje'
	            )
	        )
	    );
	    
	}

	function sub_cancel() {
	    $nonce = $_REQUEST['_wpnonce'];
	    if(!wp_verify_nonce($nonce)) {
	        die(__('Security check', 'textdomain'));
	    }
	    $id = (int) $_REQUEST['o'];
	    if(get_post_type($id) != "shop_order") wc_add_notice('Brak zamówienia', 'error');
	    else {
    	    $order = new WC_Order($id);
    	    if(get_post_meta($order->id, '_customer_user', true) != get_current_user_id()) wc_add_notice('Brak zamówienia', 'error');
    	    else {
    	        $order->update_status('completed', 'Zakończono na żądanie klienta');
        	    wc_add_notice('Subskrypcja została zakończona');
    	    }
	    }
        $this->list_my_subs();
	}

	function fdp_get_template($located, $template_name) {
	    if('subs.php' === $template_name) return plugin_dir_path(__FILE__) . 'templates/' . $template_name;
	    return $located;
	}

	public function init_methods() {
	    //TODO kontrola waluty
//	    if(!get_option('firstdata_serv')) return;
	    $this->methods[] = 'Firstdata_Recurring_Gateway';
	}

	public function woocommerce_payment_gateways($methods) {
		foreach($this->methods as $method) {
			$methods[] = $method;
		}
		return $methods;
	}

	public function woocommerce_available_payment_gateways($gateways) {
	    if(!Firstdata_Recurring_Gateway::have_jpw()) return $gateways;

	    if(((isset($_GET['pay_for_order']) && isset($_GET['key'])) || isset(WC()->cart)) && is_jpw_subscribe()) {
	        if(!$gateways['firstdata_recurring']) {
	            $fdp_gateway = new Firstdata_Recurring_Gateway();
	            $fdp_gateway->init();
	            $gateways['firstdata_recurring'] = $fdp_gateway;
	        }
	    }
	    else {
	        unset( $gateways['firstdata_recurring'] );
	    }
	    return $gateways;
	}

	public function woocommerce_get_settings_pages($woocommerce_settings) {
		include('classes/settings.php');
		$settings = new Firstdata_Settings();
		$woocommerce_settings[] = $settings;
		return $woocommerce_settings;
	}
	
	public function linksFilter($links) {
		$plugin_links = array(
			'<a href="' . admin_url('admin.php?page=wc-settings&tab=firstdata_settings') . '">' . __('Settings') . '</a>'
        );
        return array_merge($plugin_links, $links);
    }

    public static function activate() {
        $timestamp = wp_next_scheduled( self::CRON_HOOK );
        if($timestamp === false) {
            wp_schedule_event(time(), 'hourly', self::CRON_HOOK);
        }
        FirstdataRecurringPlugin::s_plugin_endpoint();
        flush_rewrite_rules();
    }
    
    public static function deactivate() {
        $timestamp = wp_next_scheduled(self::CRON_HOOK);
        wp_unschedule_event($timestamp, self::CRON_HOOK);
        flush_rewrite_rules();
    }

    public function doCron() {
        error_log('doCron');
        $fdp_gateway = new Firstdata_Recurring_Gateway();
        $fdp_gateway->init();
        $fdp_gateway->doCron();
    }

}

function FirstdataRecurringPlugin() {
    return FirstdataRecurringPlugin::FirstdataRecurringPlugin();
}

$_GLOBALS['FirstdataRecurringPlugin'] = FirstdataRecurringPlugin();

register_activation_hook(__FILE__, array(FirstdataRecurringPlugin::class, 'activate'));
register_deactivation_hook(__FILE__, array(FirstdataRecurringPlugin::class, 'deactivate'));
